{% extends "registration/base.html" %}

{% load i18n %}


{% block title %}{% trans "Password Reset Complete" %}{% endblock title %}

{% block page_main_title %}{% trans "Password Reset Complete" %}{% endblock page_main_title %}

{% block content %}
    <p>{% blocktrans %}Your password has been set. You may go ahead and <a href="{{ login_url }}">log in</a> now.{% endblocktrans %}</p>
{% endblock content %}
