{% extends "registration/base.html" %}

{% load i18n %}


{% block title %}{% trans "Password Reset Successful" %}{% endblock title %}

{% block page_main_title %}{% trans "Password Reset Successful" %}{% endblock page_main_title %}

{% block content %}
    <p>{% trans "We've e-mailed you instructions for setting your password to the e-mail address you submitted. You should be receiving it shortly." %}</p>
{% endblock content %}
